@@ -1,8 +1,8 @@
-# Copyright (C) 2003, 2004, 2006, 2007  Free Software Foundation, Inc.
+# Copyright (C) 2003, 2004  Free Software Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3, or (at your option)
+# the Free Software Foundation; either version 2, or (at your option)
 # any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -11,7 +11,9 @@
 # GNU General Public License for more details.
 
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+# 02110-1301, USA.
 
 package Automake::Rule;
 use strict;
@@ -29,8 +31,7 @@ use vars '@ISA', '@EXPORT', '@EXPORT_OK';
 @ISA = qw/Automake::Item Exporter/;
 @EXPORT = qw (reset register_suffix_rule suffix_rules_count
 	      suffixes rules $suffix_rules $KNOWN_EXTENSIONS_PATTERN
-	      depend %dependencies %actions register_action
-	      accept_extensions
+	      depend %dependencies %actions accept_extensions
 	      reject_rule msg_rule msg_cond_rule err_rule err_cond_rule
 	      rule rrule ruledef rruledef);
 
@@ -61,9 +62,9 @@ and one to look up the conditional definition:
     {
       my $def = $rule->def ($cond);
       if ($def)
-	{
-	  return $def->location;
-	}
+        {
+          return $def->location;
+        }
       ...
     }
   ...
@@ -75,7 +76,7 @@ being looked up exist, the above can be simplified to
 
 but is better written
 
-  return rrule ($name)->rdef ($cond)->location;
+  return rrule ($name)->rrule ($cond)->location;
 
 or even
 
@@ -108,7 +109,7 @@ use vars '$_suffix_rules_default';
 Holds the dependencies of targets which dependencies are factored.
 Typically, C<.PHONY> will appear in plenty of F<*.am> files, but must
 be output once.  Arguably all pure dependencies could be subject to
-this factoring, but it is not unpleasant to have paragraphs in
+this factorization, but it is not unpleasant to have paragraphs in
 Makefile: keeping related stuff altogether.
 
 =cut
@@ -142,7 +143,7 @@ C<$suffix_rules{'.foo'}{'.$(OBJEXT)'}> exists.  This will work even if
 transforming C<.foo> to C<.$(OBJEXT)> involves a chain of several
 suffix rules.
 
-The value of C<$suffix_rules{$ext1}{$ext2}> is a pair
+The value of C<$suffix_rules{$ext1}{$ext2}> is the a pair
 C<[ $next_sfx, $dist ]> where C<$next_sfx> is target suffix
 for the next rule to use to reach C<$ext2>, and C<$dist> the
 distance to C<$ext2'>.
@@ -271,7 +272,7 @@ sub accept_extensions (@)
 
 =item C<rules>
 
-Return the list of all L<Automake::Rule> instances.  (I.e., all
+Returns the list of all L<Automake::Rule> instances.  (I.e., all
 rules defined so far.)
 
 =cut
@@ -283,27 +284,6 @@ sub rules ()
 }
 
 
-=item C<register_action($target, $action)>
-
-Append the C<$action> to C<$actions{$target}> taking care of special
-cases.
-
-=cut
-
-sub register_action ($$)
-{
-  my ($target, $action) = @_;
-  if ($actions{$target})
-    {
-      $actions{$target} .= "\n$action" if $action;
-    }
-  else
-    {
-      $actions{$target} = $action;
-    }
-}
-
-
 =item C<Automake::Rule::reset>
 
 The I<forget all> function.  Clears all know rules and reset some
@@ -498,10 +478,11 @@ sub rule ($)
   # Strip $(EXEEXT) from $name, so we can diagnose
   # a clash if `ctags$(EXEEXT):' is redefined after `ctags:'.
   $name =~ s,\$\(EXEEXT\)$,,;
-  return $_rule_dict{$name} || 0;
+  return $_rule_dict{$name} if exists $_rule_dict{$name};
+  return 0;
 }
 
-=item C<ruledef ($rulename, $cond)>
+=item C<rule ($rulename, $cond>
 
 Return the C<Automake::RuleDef> object for the rule named
 C<$rulename> if defined in condition C<$cond>.  Return false
@@ -574,7 +555,7 @@ sub _new ($$)
 }
 
 
-=item C<@conds = define ($rulename, $source, $owner, $cond, $where)>
+=itcem C<@conds = define ($rulename, $source, $owner, $cond, $where)>
 
 Define a new rule.  C<$rulename> is the list of targets.  C<$source>
 is the filename the rule comes from.  C<$owner> is the owner of the
@@ -650,7 +631,7 @@ sub define ($$$$$)
 	      if ($target !~ /^[^%]*%[^%]*$/)
 		{
 		  ## FIXME: Presently we can't diagnose duplicate user rules
-		  ## because we don't distinguish rules with commands
+		  ## because we doesn't distinguish rules with commands
 		  ## from rules that only add dependencies.  E.g.,
 		  ##   .PHONY: foo
 		  ##   .PHONY: bar
@@ -659,11 +640,11 @@ sub define ($$$$$)
 		  # msg ('syntax', $where,
 		  #      "redefinition of `$target'$condmsg...", partial => 1);
 		  # msg_cond_rule ('syntax', $cond, $target,
-		  #		   "... `$target' previously defined here");
+		  # 		   "... `$target' previously defined here");
 		}
 	      # Return so we don't redefine the rule in our tables,
 	      # don't check for ambiguous condition, etc.  The rule
-	      # will be output anyway because &read_am_file ignore the
+	      # will be output anyway beauce &read_am_file ignore the
 	      # return code.
 	      return ();
 	    }
@@ -708,7 +689,7 @@ sub define ($$$$$)
 	      # Automake should ignore redefinitions of its own
 	      # rules if they came from the same file.  This makes
 	      # it easier to process a Makefile fragment several times.
-	      # However it's an error if the target is defined in many
+	      # Hower it's an error if the target is defined in many
 	      # files.  E.g., the user might be using bin_PROGRAMS = ctags
 	      # which clashes with our `ctags' rule.
 	      # (It would be more accurate if we had a way to compare
@@ -799,7 +780,7 @@ sub define ($$$$$)
       # Check if the rule is a suffix rule: either it's a rule for
       # two known extensions...
       if ($t =~ /^($KNOWN_EXTENSIONS_PATTERN)($KNOWN_EXTENSIONS_PATTERN)$/
-	  # ...or it's a rule with unknown extensions (i.e., the rule
+	  # ...or it's a rule with unknown extensions (.i.e, the rule
 	  # looks like `.foo.bar:' but `.foo' or `.bar' are not
 	  # declared in SUFFIXES and are not known language
 	  # extensions).  Automake will complete SUFFIXES from
